 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                                                                       ;;
 ;;                Centre for Speech Technology Research                  ;;
 ;;                     University of Edinburgh, UK                       ;;
 ;;                       Copyright (c) 1996,1997                         ;;
 ;;                        All Rights Reserved.                           ;;
 ;;                                                                       ;;
 ;;  Permission to use, copy, modify, distribute this software and its    ;;
 ;;  documentation for research, educational and individual use only, is  ;;
 ;;  hereby granted without fee, subject to the following conditions:     ;;
 ;;   1. The code must retain the above copyright notice, this list of    ;;
 ;;      conditions and the following disclaimer.                         ;;
 ;;   2. Any modifications must be clearly marked as such.                ;;
 ;;   3. Original authors' names are not deleted.                         ;;
 ;;  This software may not be used for commercial purposes without        ;;
 ;;  specific prior written permission from the authors.                  ;;
 ;;                                                                       ;;
 ;;  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        ;;
 ;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
 ;;  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   ;;
 ;;  SHALL THE UNIVERSITY OF EDINBURGH NOR THE CONTRIBUTORS BE LIABLE     ;;
 ;;  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    ;;
 ;;  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   ;;
 ;;  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          ;;
 ;;  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       ;;
 ;;  THIS SOFTWARE.                                                       ;;
 ;;                                                                       ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                                                                       ;;
 ;;                 Author: Richard Caley (rjc@cstr.ed.ac.uk)             ;;
 ;; -------------------------------------------------------------------   ;;
 ;; Tower of Hanoi solver for festival.                                   ;;
 ;;                                                                       ;;
 ;; This solves the puzzle recursively rather than useing the             ;;
 ;; itterative algorithm.                                                 ;;
 ;;                                                                       ;;
 ;; It also no doubt show up how bad I am at Scheme.                      ;;
 ;;                                                                       ;;
 ;; Call as                                                               ;;
 ;;         (hanoi 3)                                                     ;;
 ;; to enjoy this mind numbing spectacle.                                 ;;
 ;;                                                                       ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defvar hanoi::quiet nil
  "hanoi::silent
  Set true to shut the hanoi solver up."
  )


;; there must be a schemier way to create the initial list of disks.

(define (hanoi::create n)
  "(hanoi::create N)
  Create an instance of the Tower of Hanoi puzzle for N disks."

  (let ((stack1 ())
	(i n))
    (while (>= i 1)
	   (set! stack1 (cons i stack1))
	   (set! i (- i 1))
	   )
    (list n stack1 () ())
    )
  )

(define (hanoi::rev l)
  "(hanoi::rev L)
  Return reverse of L."
  (if (null? l)
      ()
      (append (hanoi::rev (cdr l)) (list (car l)))
      )
  )

(define (hanoi::display puzzle)
  "(hanoi::display PUZZLE)
  Print a simple representation of the given puzzle state"

  (puts "----\n")
  (puts "|")
  (mapcar (lambda (x) (puts x) (puts " ")) (hanoi::rev (cadr puzzle)))
  (puts "\n")
  (puts "|")
  (mapcar (lambda (x) (puts x) (puts " ")) (hanoi::rev (caddr puzzle)))
  (puts "\n")
  (puts "|")
  (mapcar (lambda (x) (puts x) (puts " ")) (hanoi::rev (caddr (cdr puzzle))))
  (puts "\n")
  (puts "----\n")
  )

(define (hanoi::change-stack i stack source dest disk)
  "(hanoi::change-stack I STACK SOURCE DEST DISK)
  STACK is the Ith stack of disks, do whjatever needs to be done
  for the move described by SOURCE DEST DISK"

  (cond
   ((equal? i source)
    (cdr stack))
   ((equal? i dest)
    (cons disk stack))
   (t
    stack)
   )
  )

(define (hanoi::move-disk puzzle source dest)
  "(hanoi::move-disk PUZZLE SOURCE DEST)
  Move top disk from SOURCE to DEST"

  
  (let ( disk n s1 s2 s3 p message)
    (set! disk (caar (nth_cdr source puzzle)))
    (set! n (car puzzle))
    (set! s1 (hanoi::change-stack 1 (cadr puzzle) source dest disk))
    (set! s2 (hanoi::change-stack 2 (caddr puzzle) source dest disk))
    (set! s3 (hanoi::change-stack 3 (caddr (cdr puzzle)) source dest disk))
    (set! p (list n s1 s2 s3))

    (set! message (string-append "Move disk " disk " from stack " source " to stack " dest ".\n"))
    (puts message)
    (if (not hanoi::quiet)
	(SayText message)
	)

    (hanoi::display p)
    p
    )
  )

(define (hanoi::free-peg source dest)
  "(hanoi::free-peg SOURCE DEST)
  Return the peg which is not SOURCE or DEST."
  (car (remove source (remove dest '(1 2 3))))
  )


(define (hanoi::move-stack puzzle n source dest)
  "(hanoi::move-stack PUZZLE N SOURCE DEST)
  Move N disks from SOURCE to DEST"

  (if (equal? n 1)
      (hanoi::move-disk puzzle source dest)
      (let (peg p1 p2 p3)

	(set! peg (hanoi::free-peg source dest))
	(set! p1 (hanoi::move-stack puzzle (- n 1) source peg))
	(set! p2 (hanoi::move-stack p1 1 source dest))
	(set! p3 (hanoi::move-stack p2 (- n 1) peg dest))
	
	p3
	)
      )
  )

(define (hanoi n)

  "(hanoid N)
  Solve the Tower of Hanoi puzzle for N disks."

  (let
      ((puzzle (hanoi::create n)))

    (hanoi::display puzzle)
    (hanoi::move-stack puzzle n 1 3)
    )

)

(provide 'hanoi)



